@@ -39,6 +39,11 @@ class EncryptedFieldMixin(models.Field):
     """A field that encrypts values with AES 256 symmetric encryption,
     using Pycryptodome.
 
+    Note: Be careful not to change/alter a pre-existing regular django field to be an
+    EncryptedField. The data for existing rows will be unencrypted in the database and
+    appear 'corrupted' when trying to decrypt/fetch it.
+    Instead, add the new EncryptedField to the model and do a data-migration
+    to transfer data from the old field.
     """
 
     def __init__(self, *args, **kwargs):
@@ -75,6 +80,10 @@ class EncryptedFieldMixin(models.Field):
 
     def decrypt(self, value):
         nonce = value[:16]
+        # Perform same nonce checks here as Pycryptodome, so we can raise a more user
+        # friendly error message
+        if not isinstance(nonce, (bytes, bytearray, memoryview)) or len(nonce) != 16:
+            raise ValueError("Data is corrupted.")
         tag = value[16:32]
         cypher_text = value[32:]
         counter = 0
@@ -89,12 +98,20 @@ class EncryptedFieldMixin(models.Field):
                 counter += 1
                 continue
             return plaintext.decode()
-        raise ValueError("AES Key incorrect or message corrupted")
+        raise ValueError("AES Key incorrect or data is corrupted")
 
     def get_internal_type(self):
         return self._internal_type
 
     def get_db_prep_save(self, value, connection):
+        if self.empty_strings_allowed and value == bytes():
+            # This tackles a corner case effecting string-based (eg Char/Text) fields
+            # during migrations when null=False, blank=True and no default has been declared.
+            # Because we are a BinaryField, a value of bytes() is provided instead of str().
+            # See https://github.com/django/django/blob/2.2.16/django/db/backends/base/schema.py#L215
+            # We change this to "" to prevent str(bytes()), ie a literal "b''" being
+            # used as the default to populate pre-existing records.
+            value = ""
         value = super().get_db_prep_save(value, connection)
         if value is not None:
             encrypted_value = self.encrypt(value)
@@ -217,6 +234,7 @@ class SearchField(models.CharField):
 
     Notes:
          Do not use model.objects.update() unless you update both the SearchField and the associated EncryptedField.
+         Always add a SearchField to a model, don't change/alter an existing regular django field.
     """
 
     description = "A secure SearchField to accompany an EncryptedField"
@@ -226,12 +244,16 @@ class SearchField(models.CharField):
         if hash_key is None:
             raise ImproperlyConfigured("you must supply a hash_key")
         self.hash_key = hash_key
+
         if encrypted_field_name is None:
             raise ImproperlyConfigured(
                 "you must supply the name of the accompanying Encrypted Field"
                 " that will hold the data"
             )
+        if not isinstance(encrypted_field_name, str):
+            raise ImproperlyConfigured("'encrypted_field_name' must be a string")
         self.encrypted_field_name = encrypted_field_name
+
         if kwargs.get("primary_key"):
             raise ImproperlyConfigured(
                 "{} does not support primary_key=True.".format(self.__class__.__name__)
@@ -265,10 +287,16 @@ class SearchField(models.CharField):
         return SEARCH_HASH_PREFIX + hashlib.sha256(v.encode()).hexdigest()
 
     def formfield(self, **kwargs):
-        """Use formfield from self.encrypted_field_name,
-        but using this field's verbose_name as the label.
+        """Use formfield from self.encrypted_field_name, passing kwargs along
+        (eg custom widget/help_text/label kwargs).
 
-        If called by Admin Panel then change to appropriate widget.
+        if 'label' is not in kwargs then add this field's 'verbose_name' as 'label',
+        otherwise the default 'verbose_name' of the encrypted field will be used in
+        the widget.
+
+        If called by Admin Panel, with the default widget ('AdminTextInputWidget'),
+        in kwargs, then change to appropriate default admin widget for the encrypted
+        field class.
         """
         defaults = {}
         if kwargs.get("label") is None:
@@ -299,7 +327,11 @@ class SearchField(models.CharField):
         return self.model._meta.get_field(self.encrypted_field_name).formfield(**kwargs)
 
     def clean(self, value, model_instance):
-        """Validate value against the validators from self.encrypted_field_name."""
+        """Validate value against the validators from self.encrypted_field_name.
+        Any validators on SearchField will be ignored.
+
+        SearchField's 'max_length' constraint will still be enforced at the database
+        level, but applied to the saved hash value."""
         return model_instance._meta.get_field(self.encrypted_field_name).clean(
             value, model_instance
         )
@@ -317,7 +349,8 @@ def get_prep_lookup_error(self):
 
 
 for name, lookup in models.Field.class_lookups.items():
-    """Register inappropriate lookups with our error handler"""
+    """Register inappropriate lookups with our error handler.
+    We allow 'isnull' for EncryptedField and 'isnull' and 'exact' for SearchField."""
     # Dynamically create classes that inherit from the right lookups
     if name != "isnull":
         lookup_class = type(
